[llvm-commits] CVS: llvm/lib/Transforms/Scalar/ADCE.cpp

Chris Lattner lattner at cs.uiuc.edu
Tue Jun 24 16:50:01 PDT 2003


Changes in directory llvm/lib/Transforms/Scalar:

ADCE.cpp updated: 1.54 -> 1.55

---
Log message:

Do not mark ALL terminators live if any instruciton in the block is live.  We only
want to mark it live if it is an unconditional branch.  This fixes bug:
ADCE/2002-05-28-Crash.ll and makes this pass _much_ more useful.



---
Diffs of the changes:

Index: llvm/lib/Transforms/Scalar/ADCE.cpp
diff -u llvm/lib/Transforms/Scalar/ADCE.cpp:1.54 llvm/lib/Transforms/Scalar/ADCE.cpp:1.55
--- llvm/lib/Transforms/Scalar/ADCE.cpp:1.54	Mon Jun 16 07:10:45 2003
+++ llvm/lib/Transforms/Scalar/ADCE.cpp	Tue Jun 24 16:49:45 2003
@@ -105,8 +105,11 @@
              bind_obj(this, &ADCE::markTerminatorLive));
   }
   
-  // If this basic block is live, then the terminator must be as well!
-  markTerminatorLive(BB);
+  // If this basic block is live, and it ends in an unconditional branch, then
+  // the branch is alive as well...
+  if (BranchInst *BI = dyn_cast<BranchInst>(BB->getTerminator()))
+    if (BI->isUnconditional())
+      markTerminatorLive(BB);
 }
 
 // dropReferencesOfDeadInstructionsInLiveBlock - Loop over all of the
@@ -248,6 +251,7 @@
       NewEntry->getInstList().push_back(new BranchInst(&Func->front()));
       Func->getBasicBlockList().push_front(NewEntry);
       AliveBlocks.insert(NewEntry);    // This block is always alive!
+      LiveSet.insert(NewEntry->getTerminator());  // The branch is live
     }
     
     // Loop over all of the alive blocks in the function.  If any successor
@@ -260,6 +264,23 @@
         BasicBlock *BB = I;
         TerminatorInst *TI = BB->getTerminator();
       
+        // If the terminator instruction is alive, but the block it is contained
+        // in IS alive, this means that this terminator is a conditional branch
+        // on a condition that doesn't matter.  Make it an unconditional branch
+        // to ONE of the successors.  This has the side effect of dropping a use
+        // of the conditional value, which may also be dead.
+        if (!LiveSet.count(TI)) {
+          assert(TI->getNumSuccessors() > 1 && "Not a conditional?");
+          BranchInst *NB = new BranchInst(TI->getSuccessor(0), TI);
+
+          // Remove entries from PHI nodes to avoid confusing ourself later...
+          for (unsigned i = 1, e = TI->getNumSuccessors(); i != e; ++i)
+              TI->getSuccessor(i)->removePredecessor(BB);
+
+          BB->getInstList().erase(TI);
+          TI = NB;
+        }
+
         // Loop over all of the successors, looking for ones that are not alive.
         // We cannot save the number of successors in the terminator instruction
         // here because we may remove them if we don't have a postdominator...





More information about the llvm-commits mailing list